@@ -14,15 +14,16 @@ at the top-level directory.
  * \brief  Distributed SuperLU data types and function prototypes
  *
  * <pre>
- * -- Distributed SuperLU routine (version 4.1) --
+ * -- Distributed SuperLU routine (version 6.0) --
  * Lawrence Berkeley National Lab, Univ. of California Berkeley.
  * November 1, 2007
  * April 5, 2015
+ * September 18, 2018  version 6.0
  * </pre>
  */
 
-#ifndef __SUPERLU_dDEFS /* allow multiple inclusions */
-#define __SUPERLU_dDEFS
+#ifndef __SUPERLU_DDEFS /* allow multiple inclusions */
+#define __SUPERLU_DDEFS
 
 /*
  * File name:	superlu_ddefs.h
@@ -45,9 +46,18 @@ typedef struct {
 #define MAX_LOOKAHEADS 50
 typedef struct {
     int_t   **Lrowind_bc_ptr; /* size ceil(NSUPERS/Pc)                 */
-    double  **Lnzval_bc_ptr;  /* size ceil(NSUPERS/Pc)                 */
+    double **Lnzval_bc_ptr;  /* size ceil(NSUPERS/Pc)                 */
+    double **Linv_bc_ptr;  /* size ceil(NSUPERS/Pc)                 */
+    int_t   **Lindval_loc_bc_ptr; /* size ceil(NSUPERS/Pc)  pointers to locations in Lrowind_bc_ptr and Lnzval_bc_ptr */
+    int_t   **Lrowind_bc_2_lsum; /* size ceil(NSUPERS/Pc)  map indices of Lrowind_bc_ptr to indices of lsum  */  
+    double  **Uinv_bc_ptr;  /* size ceil(NSUPERS/Pc)     	*/
     int_t   **Ufstnz_br_ptr;  /* size ceil(NSUPERS/Pr)                 */
     double  **Unzval_br_ptr;  /* size ceil(NSUPERS/Pr)                 */
+        /*-- Data structures used for broadcast and reduction trees. --*/
+    BcTree  *LBtree_ptr;       /* size ceil(NSUPERS/Pc)                */
+    RdTree  *LRtree_ptr;       /* size ceil(NSUPERS/Pr)                */
+    BcTree  *UBtree_ptr;       /* size ceil(NSUPERS/Pc)                */
+    RdTree  *URtree_ptr;       /* size ceil(NSUPERS/Pr)			*/
 #if 0
     int_t   *Lsub_buf;        /* Buffer for the remote subscripts of L */
     double  *Lval_buf;        /* Buffer for the remote nonzeros of L   */
@@ -118,6 +128,7 @@ typedef struct {
     int_t n;
     int_t nleaf;
     int_t nfrecvmod;
+    int_t inv; /* whether the diagonal block is inverted*/	
 } LocalLU_t;
 
 
@@ -211,7 +222,10 @@ extern int     dcreate_matrix_rb(SuperMatrix *, int, double **, int *,
 			      double **, int *, FILE *, gridinfo_t *);
 extern int     dcreate_matrix_dat(SuperMatrix *, int, double **, int *, 
 			      double **, int *, FILE *, gridinfo_t *);
-
+extern int 	   dcreate_matrix_postfix(SuperMatrix *, int, double **, int *, 
+				  double **, int *, FILE *, char *, gridinfo_t *);				  
+				  
+	
 /* Driver related */
 extern void    dgsequ_dist (SuperMatrix *, double *, double *, double *,
 			    double *, double *, int_t *);
@@ -247,6 +261,7 @@ extern void  pdgssvx(superlu_dist_options_t *, SuperMatrix *,
 		     ScalePermstruct_t *, double *,
 		     int, int, gridinfo_t *, LUstruct_t *,
 		     SOLVEstruct_t *, double *, SuperLUStat_t *, int *);
+extern void  pdCompute_Diag_Inv(int_t, LUstruct_t *,gridinfo_t *, SuperLUStat_t *, int *);
 extern int  dSolveInit(superlu_dist_options_t *, SuperMatrix *, int_t [], int_t [],
 		       int_t, LUstruct_t *, gridinfo_t *, SOLVEstruct_t *);
 extern void dSolveFinalize(superlu_dist_options_t *, SOLVEstruct_t *);
@@ -262,6 +277,7 @@ extern int  static_schedule(superlu_dist_options_t *, int, int,
 extern void LUstructInit(const int_t, LUstruct_t *);
 extern void LUstructFree(LUstruct_t *);
 extern void Destroy_LU(int_t, gridinfo_t *, LUstruct_t *);
+extern void dDestroy_Tree(int_t, gridinfo_t *, LUstruct_t *);
 
 /* #define GPU_PROF
 #define IPM_PROF */
@@ -281,6 +297,24 @@ extern void dlsum_bmod(double *, double *, double *,
                        int, int_t, int_t *, int_t *, Ucb_indptr_t **,
                        int_t **, int_t *, gridinfo_t *, LocalLU_t *,
 		       MPI_Request [], SuperLUStat_t *);
+
+extern void dlsum_fmod_inv(double *, double *, double *, double *,
+		       int, int, int_t , int_t *, int_t,
+		       int_t *, gridinfo_t *, LocalLU_t *, 
+		       SuperLUStat_t **, int_t *, int_t *, int_t, int_t, int_t);
+extern void dlsum_fmod_inv_master(double *, double *, double *, double *,
+		       int, int, int_t , int_t *, int_t, 
+		       int_t *, gridinfo_t *, LocalLU_t *, 
+		       SuperLUStat_t **, int_t, int_t, int_t);
+extern void dlsum_bmod_inv(double *, double *, double *, double *,
+                       int, int_t, int_t *, int_t *, int_t *, Ucb_indptr_t **,
+                       int_t **, int_t *, gridinfo_t *, LocalLU_t *,
+		       MPI_Request [], SuperLUStat_t **, int_t *, int_t *, int_t, int_t);
+extern void dlsum_bmod_inv_master(double *, double *, double *, double *,
+                       int, int_t, int_t *, int_t *, int_t *, Ucb_indptr_t **,
+                       int_t **, int_t *, gridinfo_t *, LocalLU_t *,
+		       MPI_Request [], SuperLUStat_t **, int_t, int_t);				   
+			   
 extern void pdgsrfs(int_t, SuperMatrix *, double, LUstruct_t *,
 		    ScalePermstruct_t *, gridinfo_t *,
 		    double [], int_t, double [], int_t, int,
@@ -310,6 +344,12 @@ extern int_t dQuerySpace_dist(int_t, LUstruct_t *, gridinfo_t *,
 			      SuperLUStat_t *, superlu_dist_mem_usage_t *);
 
 /* Auxiliary routines */
+
+extern void dClone_CompRowLoc_Matrix_dist(SuperMatrix *, SuperMatrix *);
+extern void dCopy_CompRowLoc_Matrix_dist(SuperMatrix *, SuperMatrix *);
+extern void dZero_CompRowLoc_Matrix_dist(SuperMatrix *);
+extern void dScaleAddId_CompRowLoc_Matrix_dist(SuperMatrix *, double);
+extern void dScaleAdd_CompRowLoc_Matrix_dist(SuperMatrix *, SuperMatrix *, double);
 extern void    dfill_dist (double *, int_t, double);
 extern void    dinf_norm_error_dist (int_t, int_t, double*, int_t,
                                      double*, int_t, gridinfo_t*);
@@ -323,7 +363,9 @@ extern void  dreadrb_dist(int, FILE *, int_t *, int_t *, int_t *,
 		     double **, int_t **, int_t **);
 extern void  dreadMM_dist(FILE *, int_t *, int_t *, int_t *,
 	                  double **, int_t **, int_t **);
-
+extern int  dread_binary(FILE *, int_t *, int_t *, int_t *,
+	                  double **, int_t **, int_t **);	
+					  
 /* Distribute the data for numerical factorization */
 extern float ddist_psymbtonum(fact_t, int_t, SuperMatrix *,
                                 ScalePermstruct_t *, Pslu_freeable_t *, 
@@ -339,6 +381,7 @@ extern void  dPrintUblocks(int, int_t, gridinfo_t *, Glu_persist_t *,
 extern void  dPrint_CompCol_Matrix_dist(SuperMatrix *);
 extern void  dPrint_Dense_Matrix_dist(SuperMatrix *);
 extern int   dPrint_CompRowLoc_Matrix_dist(SuperMatrix *);
+extern int   file_dPrint_CompRowLoc_Matrix_dist(FILE *fp, SuperMatrix *A);																			   
 extern int   file_PrintDouble5(FILE *, char *, int_t, double *);
 
 
@@ -355,6 +398,8 @@ extern void dtrsm_(char*, char*, char*, char*, int*, int*,
                   int*, int, int, int, int);
 extern void dgemv_(char *, int *, int *, double *, double *a, int *, 
                   double *, int *, double *, double *, int *, int);
+extern void dtrtri_(char*, char*, int*, double*, int*,int*);				 
+
 extern void dger_(int*, int*, double*, double*, int*,
                  double*, int*, double*, int*);
 
