@@ -13,15 +13,16 @@ at the top-level directory.
  * \brief  Distributed SuperLU data types and function prototypes
  *
  * <pre>
- * -- Distributed SuperLU routine (version 4.1) --
+ * -- Distributed SuperLU routine (version 6.0) --
  * Lawrence Berkeley National Lab, Univ. of California Berkeley.
  * November 1, 2007
  * April 5, 2015
+ * September 18, 2018  version 6.0
  * </pre>
  */
 
-#ifndef __SUPERLU_zDEFS /* allow multiple inclusions */
-#define __SUPERLU_zDEFS
+#ifndef __SUPERLU_ZDEFS /* allow multiple inclusions */
+#define __SUPERLU_ZDEFS
 
 /*
  * File name:	superlu_zdefs.h
@@ -45,9 +46,18 @@ typedef struct {
 #define MAX_LOOKAHEADS 50
 typedef struct {
     int_t   **Lrowind_bc_ptr; /* size ceil(NSUPERS/Pc)                 */
-    doublecomplex  **Lnzval_bc_ptr;  /* size ceil(NSUPERS/Pc)                 */
+    doublecomplex **Lnzval_bc_ptr;  /* size ceil(NSUPERS/Pc)                 */
+    doublecomplex **Linv_bc_ptr;  /* size ceil(NSUPERS/Pc)                 */
+    int_t   **Lindval_loc_bc_ptr; /* size ceil(NSUPERS/Pc)  pointers to locations in Lrowind_bc_ptr and Lnzval_bc_ptr */
+    int_t   **Lrowind_bc_2_lsum; /* size ceil(NSUPERS/Pc)  map indices of Lrowind_bc_ptr to indices of lsum  */  
+    doublecomplex  **Uinv_bc_ptr;  /* size ceil(NSUPERS/Pc)     	*/
     int_t   **Ufstnz_br_ptr;  /* size ceil(NSUPERS/Pr)                 */
     doublecomplex  **Unzval_br_ptr;  /* size ceil(NSUPERS/Pr)                 */
+        /*-- Data structures used for broadcast and reduction trees. --*/
+    BcTree  *LBtree_ptr;       /* size ceil(NSUPERS/Pc)                */
+    RdTree  *LRtree_ptr;       /* size ceil(NSUPERS/Pr)                */
+    BcTree  *UBtree_ptr;       /* size ceil(NSUPERS/Pc)                */
+    RdTree  *URtree_ptr;       /* size ceil(NSUPERS/Pr)			*/
 #if 0
     int_t   *Lsub_buf;        /* Buffer for the remote subscripts of L */
     double  *Lval_buf;        /* Buffer for the remote nonzeros of L   */
@@ -118,6 +128,7 @@ typedef struct {
     int_t n;
     int_t nleaf;
     int_t nfrecvmod;
+    int_t inv; /* whether the diagonal block is inverted*/	
 } LocalLU_t;
 
 
@@ -211,7 +222,10 @@ extern int     zcreate_matrix_rb(SuperMatrix *, int, doublecomplex **, int *,
 			      doublecomplex **, int *, FILE *, gridinfo_t *);
 extern int     zcreate_matrix_dat(SuperMatrix *, int, doublecomplex **, int *, 
 			      doublecomplex **, int *, FILE *, gridinfo_t *);
-
+extern int 	   zcreate_matrix_postfix(SuperMatrix *, int, doublecomplex **, int *, 
+				  doublecomplex **, int *, FILE *, char *, gridinfo_t *);				  
+				  
+	
 /* Driver related */
 extern void    zgsequ_dist (SuperMatrix *, double *, double *, double *,
 			    double *, double *, int_t *);
@@ -247,6 +261,7 @@ extern void  pzgssvx(superlu_dist_options_t *, SuperMatrix *,
 		     ScalePermstruct_t *, doublecomplex *,
 		     int, int, gridinfo_t *, LUstruct_t *,
 		     SOLVEstruct_t *, double *, SuperLUStat_t *, int *);
+extern void  pzCompute_Diag_Inv(int_t, LUstruct_t *,gridinfo_t *, SuperLUStat_t *, int *);
 extern int  zSolveInit(superlu_dist_options_t *, SuperMatrix *, int_t [], int_t [],
 		       int_t, LUstruct_t *, gridinfo_t *, SOLVEstruct_t *);
 extern void zSolveFinalize(superlu_dist_options_t *, SOLVEstruct_t *);
@@ -262,6 +277,7 @@ extern int  static_schedule(superlu_dist_options_t *, int, int,
 extern void LUstructInit(const int_t, LUstruct_t *);
 extern void LUstructFree(LUstruct_t *);
 extern void Destroy_LU(int_t, gridinfo_t *, LUstruct_t *);
+extern void zDestroy_Tree(int_t, gridinfo_t *, LUstruct_t *);
 
 /* #define GPU_PROF
 #define IPM_PROF */
@@ -281,6 +297,24 @@ extern void zlsum_bmod(doublecomplex *, doublecomplex *, doublecomplex *,
                        int, int_t, int_t *, int_t *, Ucb_indptr_t **,
                        int_t **, int_t *, gridinfo_t *, LocalLU_t *,
 		       MPI_Request [], SuperLUStat_t *);
+
+extern void zlsum_fmod_inv(doublecomplex *, doublecomplex *, doublecomplex *, doublecomplex *,
+		       int, int, int_t , int_t *, int_t,
+		       int_t *, gridinfo_t *, LocalLU_t *, 
+		       SuperLUStat_t **, int_t *, int_t *, int_t, int_t, int_t);
+extern void zlsum_fmod_inv_master(doublecomplex *, doublecomplex *, doublecomplex *, doublecomplex *,
+		       int, int, int_t , int_t *, int_t, 
+		       int_t *, gridinfo_t *, LocalLU_t *, 
+		       SuperLUStat_t **, int_t, int_t, int_t);
+extern void zlsum_bmod_inv(doublecomplex *, doublecomplex *, doublecomplex *, doublecomplex *,
+                       int, int_t, int_t *, int_t *, int_t *, Ucb_indptr_t **,
+                       int_t **, int_t *, gridinfo_t *, LocalLU_t *,
+		       MPI_Request [], SuperLUStat_t **, int_t *, int_t *, int_t, int_t);
+extern void zlsum_bmod_inv_master(doublecomplex *, doublecomplex *, doublecomplex *, doublecomplex *,
+                       int, int_t, int_t *, int_t *, int_t *, Ucb_indptr_t **,
+                       int_t **, int_t *, gridinfo_t *, LocalLU_t *,
+		       MPI_Request [], SuperLUStat_t **, int_t, int_t);				   
+			   
 extern void pzgsrfs(int_t, SuperMatrix *, double, LUstruct_t *,
 		    ScalePermstruct_t *, gridinfo_t *,
 		    doublecomplex [], int_t, doublecomplex [], int_t, int,
@@ -312,6 +346,12 @@ extern int_t zQuerySpace_dist(int_t, LUstruct_t *, gridinfo_t *,
 			      SuperLUStat_t *, superlu_dist_mem_usage_t *);
 
 /* Auxiliary routines */
+
+extern void zClone_CompRowLoc_Matrix_dist(SuperMatrix *, SuperMatrix *);
+extern void zCopy_CompRowLoc_Matrix_dist(SuperMatrix *, SuperMatrix *);
+extern void zZero_CompRowLoc_Matrix_dist(SuperMatrix *);
+extern void zScaleAddId_CompRowLoc_Matrix_dist(SuperMatrix *, doublecomplex);
+extern void zScaleAdd_CompRowLoc_Matrix_dist(SuperMatrix *, SuperMatrix *, doublecomplex);
 extern void    zfill_dist (doublecomplex *, int_t, doublecomplex);
 extern void    zinf_norm_error_dist (int_t, int_t, doublecomplex*, int_t,
                                      doublecomplex*, int_t, gridinfo_t*);
@@ -325,7 +365,9 @@ extern void  zreadrb_dist(int, FILE *, int_t *, int_t *, int_t *,
 		     doublecomplex **, int_t **, int_t **);
 extern void  zreadMM_dist(FILE *, int_t *, int_t *, int_t *,
 	                  doublecomplex **, int_t **, int_t **);
-
+extern int  zread_binary(FILE *, int_t *, int_t *, int_t *,
+	                  doublecomplex **, int_t **, int_t **);	
+					  
 /* Distribute the data for numerical factorization */
 extern float zdist_psymbtonum(fact_t, int_t, SuperMatrix *,
                                 ScalePermstruct_t *, Pslu_freeable_t *, 
@@ -341,6 +383,7 @@ extern void  zPrintUblocks(int, int_t, gridinfo_t *, Glu_persist_t *,
 extern void  zPrint_CompCol_Matrix_dist(SuperMatrix *);
 extern void  zPrint_Dense_Matrix_dist(SuperMatrix *);
 extern int   zPrint_CompRowLoc_Matrix_dist(SuperMatrix *);
+extern int   file_zPrint_CompRowLoc_Matrix_dist(FILE *fp, SuperMatrix *A);																			   
 extern void  PrintDoublecomplex(char *, int_t, doublecomplex *);
 extern int   file_PrintDoublecomplex(FILE *fp, char *, int_t, doublecomplex *);
 
@@ -358,6 +401,8 @@ extern void ztrsm_(char*, char*, char*, char*, int*, int*,
                   int*, int, int, int, int);
 extern void zgemv_(char *, int *, int *, doublecomplex *, doublecomplex *a, int *, 
                   doublecomplex *, int *, doublecomplex *, doublecomplex *, int *, int);
+extern void ztrtri_(char*, char*, int*, doublecomplex*, int*,int*);				 
+
 extern void zgeru_(int*, int*, doublecomplex*, doublecomplex*, int*,
                  doublecomplex*, int*, doublecomplex*, int*);
 
