@@ -22,6 +22,7 @@ at the top-level directory.
  */
 
 #include <math.h>
+#include <unistd.h>
 #include "superlu_ddefs.h"
 
 /*! \brief Deallocate the structure pointing to the actual storage of the matrix. */
@@ -144,6 +145,39 @@ Destroy_LU(int_t n, gridinfo_t *grid, LUstruct_t *LUstruct)
     SUPERLU_FREE(Llu->bsendx_plist);
     SUPERLU_FREE(Llu->mod_bit);
 
+	
+	
+    nb = CEILING(nsupers, grid->npcol);
+    for (i = 0; i < nb; ++i) 
+	if ( Llu->Lindval_loc_bc_ptr[i] ) {
+	    SUPERLU_FREE (Llu->Lindval_loc_bc_ptr[i]);
+	}	
+	SUPERLU_FREE(Llu->Lindval_loc_bc_ptr);
+	
+	nb = CEILING(nsupers, grid->npcol);
+	for (i=0;i<nb;++i){
+		if(Llu->Linv_bc_ptr[i]!=NULL){
+			SUPERLU_FREE(Llu->Linv_bc_ptr[i]);
+		}
+		if(Llu->Uinv_bc_ptr[i]!=NULL){
+			SUPERLU_FREE(Llu->Uinv_bc_ptr[i]);
+		}	
+	}
+	SUPERLU_FREE(Llu->Linv_bc_ptr);
+	SUPERLU_FREE(Llu->Uinv_bc_ptr);
+	
+	
+	nb = CEILING(nsupers, grid->npcol);
+    for (i = 0; i < nb; ++i)
+	if ( Llu->Urbs[i] ) {
+	    SUPERLU_FREE(Llu->Ucb_indptr[i]);
+	    SUPERLU_FREE(Llu->Ucb_valptr[i]);
+	}
+    SUPERLU_FREE(Llu->Ucb_indptr);
+    SUPERLU_FREE(Llu->Ucb_valptr);	
+	SUPERLU_FREE(Llu->Urbs);
+
+	
     SUPERLU_FREE(Glu_persist->xsup);
     SUPERLU_FREE(Glu_persist->supno);
 
@@ -193,6 +227,7 @@ void LUstructInit(const int_t n, LUstruct_t *LUstruct)
     if ( !(LUstruct->Llu = (LocalLU_t *)
 	   SUPERLU_MALLOC(sizeof(LocalLU_t))) )
 	ABORT("Malloc fails for LocalLU_t.");
+	LUstruct->Llu->inv = 0;
 }
 
 /*! \brief Deallocate LUstruct */
@@ -222,7 +257,7 @@ void LUstructFree(LUstruct_t *LUstruct)
  */
 void
 countnz_dist(const int_t n, int_t *xprune,
-	     long long int *nnzL, long long int *nnzU, 
+	     int_t *nnzL, int_t *nnzU, 
 	     Glu_persist_t *Glu_persist, Glu_freeable_t *Glu_freeable)
 {
     int_t  fnz, fsupc, i, j, nsuper;
@@ -279,7 +314,7 @@ countnz_dist(const int_t n, int_t *xprune,
  * subscripts.
  * </pre>
  */
-long long int
+int64_t
 fixupL_dist(const int_t n, const int_t *perm_r, 
 	    Glu_persist_t *Glu_persist, Glu_freeable_t *Glu_freeable)
 {
@@ -338,6 +373,11 @@ void set_default_options_dist(superlu_dist_options_t *options)
     options->num_lookaheads    = 10;
     options->lookahead_etree   = NO;
     options->SymPattern        = NO;
+#ifdef HAVE_LAPACK
+    options->DiagInv           = YES;
+#else
+    options->DiagInv           = NO;
+#endif
 }
 
 /*! \brief Print the options setting.
@@ -620,60 +660,121 @@ PStatPrint(superlu_dist_options_t *options, SuperLUStat_t *stat, gridinfo_t *gri
 	       0, grid->comm);
     solveflop = flopcnt;
     if ( !iam ) {
-	printf("\tSOLVE time         %8.2f\n", utime[SOLVE]);
+	printf("\tSOLVE time         %8.3f\n", utime[SOLVE]);
 	if ( utime[SOLVE] != 0.0 )
 	    printf("\tSolve flops\t%e\tMflops \t%8.2f\n",
 		   flopcnt,
 		   flopcnt*1e-6/utime[SOLVE]);
 	if ( options->IterRefine != NOREFINE ) {
-	    printf("\tREFINEMENT time    %8.2f\tSteps%8d\n\n",
+	    printf("\tREFINEMENT time    %8.3f\tSteps%8d\n\n",
 		   utime[REFINE], stat->RefineSteps);
 	}
 	printf("**************************************************\n");
     }
 
+	double  *utime1,*utime2,*utime3,*utime4;
+	flops_t  *ops1;
 #if ( PROFlevel>=1 )
-    fflush(stdout);
+	fflush(stdout);
     MPI_Barrier( grid->comm );
 
     {
 	int_t i, P = grid->nprow*grid->npcol;
 	flops_t b, maxflop;
+	
+		
+	if ( !iam )utime1=doubleMalloc_dist(P);
+	if ( !iam )utime2=doubleMalloc_dist(P);
+	if ( !iam )utime3=doubleMalloc_dist(P);
+	if ( !iam )utime4=doubleMalloc_dist(P);
+	if ( !iam )ops1=(flops_t *) SUPERLU_MALLOC(P * sizeof(flops_t));
+
+	
+	// fflush(stdout); 
+	// if ( !iam ) printf("\n.. Tree max sizes:\tbtree\trtree\n");
+	// fflush(stdout);
+	// sleep(2.0); 	
+	// MPI_Barrier( grid->comm );
+	// for (i = 0; i < P; ++i) {
+	    // if ( iam == i) {
+		// printf("\t\t%d %5d %5d\n", iam, stat->MaxActiveBTrees,stat->MaxActiveRTrees);
+		// fflush(stdout);
+	    // }
+	    // MPI_Barrier( grid->comm );
+	// }	
+	
+	// sleep(2.0); 	
+
+	
+	MPI_Barrier( grid->comm );	
+	
 	if ( !iam ) printf("\n.. FACT time breakdown:\tcomm\ttotal\n");
+
+    MPI_Gather(&utime[COMM], 1, MPI_DOUBLE,utime1, 1 , MPI_DOUBLE, 0, grid->comm);	
+    MPI_Gather(&utime[FACT], 1, MPI_DOUBLE,utime2, 1 , MPI_DOUBLE, 0, grid->comm);	
+	if ( !iam ) 
 	for (i = 0; i < P; ++i) {
-	    if ( iam == i) {
-		printf("\t\t(%d)%8.2f%8.2f\n", iam, utime[COMM], utime[FACT]);
-		fflush(stdout);
-	    }
-	    MPI_Barrier( grid->comm );
+		printf("\t\t(%d)%8.2f%8.2f\n", i, utime1[i], utime2[i]);
 	}
+	fflush(stdout);
+	MPI_Barrier( grid->comm );	
+	
 	if ( !iam ) printf("\n.. FACT ops distribution:\n");
+    MPI_Gather(&ops[FACT], 1, MPI_FLOAT,ops1, 1 , MPI_FLOAT, 0, grid->comm);
+	
+	if ( !iam ) 
 	for (i = 0; i < P; ++i) {
-	    if ( iam == i ) {
-		printf("\t\t(%d)\t%e\n", iam, ops[FACT]);
-		fflush(stdout);
-	    }
-	    MPI_Barrier( grid->comm );
+		printf("\t\t(%d)\t%e\n", i, ops1[i]);
 	}
+	fflush(stdout);
+	MPI_Barrier( grid->comm );
+	
 	MPI_Reduce(&ops[FACT], &maxflop, 1, MPI_FLOAT, MPI_MAX, 0, grid->comm);
+
 	if ( !iam ) {
 	    b = factflop/P/maxflop;
 	    printf("\tFACT load balance: %.2f\n", b);
 	}
-	if ( !iam ) printf("\n.. SOLVE ops distribution:\n");
+	fflush(stdout);
+	MPI_Barrier( grid->comm );
+
+	
+	if ( !iam ) printf("\n.. SOLVE time breakdown:\tcommL \tgemmL\ttrsmL\ttotal\n");
+
+    MPI_Gather(&utime[SOL_COMM], 1, MPI_DOUBLE,utime1, 1 , MPI_DOUBLE, 0, grid->comm);	
+    MPI_Gather(&utime[SOL_GEMM], 1, MPI_DOUBLE,utime2, 1 , MPI_DOUBLE, 0, grid->comm);		
+    MPI_Gather(&utime[SOL_TRSM], 1, MPI_DOUBLE,utime3, 1 , MPI_DOUBLE, 0, grid->comm);		
+    MPI_Gather(&utime[SOL_TOT], 1, MPI_DOUBLE,utime4, 1 , MPI_DOUBLE, 0, grid->comm);		
+	if ( !iam ) 	
 	for (i = 0; i < P; ++i) {
-	    if ( iam == i ) {
-		printf("\t\t%d\t%e\n", iam, ops[SOLVE]);
-		fflush(stdout);
-	    }
-	    MPI_Barrier( grid->comm );
+		printf("\t\t\t%d%10.5f%10.5f%10.5f%10.5f\n", i,utime1[i],utime2[i],utime3[i], utime4[i]);
+	}
+	fflush(stdout); 
+	MPI_Barrier( grid->comm );	
+	
+	if ( !iam ) printf("\n.. SOLVE ops distribution:\n"); 
+    MPI_Gather(&ops[SOLVE], 1, MPI_FLOAT,ops1, 1 , MPI_FLOAT, 0, grid->comm);	
+	if ( !iam ) 
+	for (i = 0; i < P; ++i) {
+		printf("\t\t%d\t%e\n", i, ops1[i]);
 	}
 	MPI_Reduce(&ops[SOLVE], &maxflop, 1, MPI_FLOAT, MPI_MAX, 0,grid->comm);
 	if ( !iam ) {
 	    b = solveflop/P/maxflop;
 	    printf("\tSOLVE load balance: %.2f\n", b);
+		fflush(stdout);
 	}
+	
     }
+	
+	if ( !iam ){
+	SUPERLU_FREE(utime1);
+	SUPERLU_FREE(utime2);
+	SUPERLU_FREE(utime3);
+	SUPERLU_FREE(utime4);
+	SUPERLU_FREE(ops1);
+	}
+	
 #endif
 
 /*  if ( !iam ) fflush(stdout);  CRASH THE SYSTEM pierre.  */
@@ -948,7 +1049,7 @@ void isort1(int_t N, int_t *ARRAY)
   }
 }
 
-void log_memory(long long cur_bytes, SuperLUStat_t *stat) {
+void log_memory(int64_t cur_bytes, SuperLUStat_t *stat) {
     stat->current_buffer += (float) cur_bytes;
     if (cur_bytes > 0) {
 	stat->peak_buffer = 
@@ -1142,7 +1243,6 @@ static int_t num_full_cols_U
 			 j, &iukp, &rukp, &jb, &ljb, &nsupc,
 			 iukp0, rukp0, usub, perm_u, xsup, grid
 			 );
-
         for (int_t jj = iukp; jj < iukp + nsupc; ++jj) {
             segsize = klst - usub[jj];
             if ( segsize ) ++temp_ncols;
@@ -1184,8 +1284,108 @@ int_t estimate_bigu_size(int_t nsupers,
     MPI_Allreduce(&ldu, &max_ldu, 1, mpi_int_t, MPI_MAX, grid->cscp.comm);
 
 #if ( PRNTlevel>=1 )
-    printf("max_ncols %d, max_ldu %d, ldt %d, bigu_size=%d\n",
+	if(iam==0)
+    printf("max_ncols " IFMT ", max_ldu " IFMT ", ldt " IFMT ", bigu_size " IFMT "\n",
 	   max_ncols, max_ldu, ldt, max_ldu*max_ncols);
 #endif
     return(max_ldu * max_ncols);
 }
+
+void quickSort( int_t* a, int_t l, int_t r, int_t dir)
+{
+   int_t j;
+
+   if( l < r ) 
+   {
+   	// divide and conquer
+       j = partition( a, l, r, dir);
+       quickSort( a, l, j-1, dir);
+       quickSort( a, j+1, r, dir);
+   }
+	
+}
+
+int_t partition( int_t* a, int_t l, int_t r, int_t dir) {
+   int_t pivot, i, j, t;
+   pivot = a[l];
+   i = l; j = r+1;
+   
+   if(dir==0){		
+	   while( 1)
+	   {
+		do ++i; while( a[i] <= pivot && i <= r );
+		do --j; while( a[j] > pivot );
+		if( i >= j ) break;
+		t = a[i]; a[i] = a[j]; a[j] = t;
+	   }
+	   t = a[l]; a[l] = a[j]; a[j] = t;
+	   return j;
+   }else if(dir==1){
+	   while( 1)
+	   {
+		do ++i; while( a[i] >= pivot && i <= r );
+		do --j; while( a[j] < pivot );
+		if( i >= j ) break;
+		t = a[i]; a[i] = a[j]; a[j] = t;
+	   }
+	   t = a[l]; a[l] = a[j]; a[j] = t;
+	   return j;	   
+   }
+}
+
+
+
+void quickSortM( int_t* a, int_t l, int_t r, int_t lda, int_t dir, int_t dims)
+{
+   int_t j;
+
+   if( l < r ) 
+   {
+	   	// printf("dims: %5d",dims);
+		// fflush(stdout);
+		
+   	// divide and conquer
+       j = partitionM( a, l, r,lda,dir, dims);
+       quickSortM( a, l, j-1,lda,dir,dims);
+       quickSortM( a, j+1, r,lda,dir,dims);
+   }
+	
+}
+
+
+int_t partitionM( int_t* a, int_t l, int_t r, int_t lda, int_t dir, int_t dims) {
+   int_t pivot, i, j, t, dd;
+   pivot = a[l];
+   i = l; j = r+1;
+
+	if(dir==0){
+	   while( 1)
+	   {
+		do ++i; while( a[i] <= pivot && i <= r );
+		do --j; while( a[j] > pivot );
+		if( i >= j ) break; 
+		for(dd=0;dd<dims;dd++){	
+			t = a[i+lda*dd]; a[i+lda*dd] = a[j+lda*dd]; a[j+lda*dd] = t;	
+		}
+	   }
+	   for(dd=0;dd<dims;dd++){	
+		t = a[l+lda*dd]; a[l+lda*dd] = a[j+lda*dd]; a[j+lda*dd] = t;
+	   }	   
+	   return j;		
+	}else if(dir==1){
+	   while( 1)
+	   {
+		do ++i; while( a[i] >= pivot && i <= r );
+		do --j; while( a[j] < pivot );
+		if( i >= j ) break;
+		for(dd=0;dd<dims;dd++){	
+			t = a[i+lda*dd]; a[i+lda*dd] = a[j+lda*dd]; a[j+lda*dd] = t;	
+		}
+	   }
+	   for(dd=0;dd<dims;dd++){	
+		t = a[l+lda*dd]; a[l+lda*dd] = a[j+lda*dd]; a[j+lda*dd] = t;
+	   } 
+	   return j;		
+	}
+}
+
