@@ -55,6 +55,7 @@ pdcompute_resid(int m, int n, int nrhs, SuperMatrix *A,
 		double *x, int ldx, double *b, int ldb,
 		gridinfo_t *grid, SOLVEstruct_t *SOLVEstruct, double *resid);
 
+#if 0
 /*! \brief Copy matrix A into matrix B, in distributed compressed row format. */
 void
 dCopy_CompRowLoc_Matrix_dist(SuperMatrix *A, SuperMatrix *B)
@@ -79,6 +80,7 @@ dCopy_CompRowLoc_Matrix_dist(SuperMatrix *A, SuperMatrix *B)
     memcpy(Bstore->colind, Astore->colind, nnz_loc * sizeof(int_t));
     memcpy(Bstore->rowptr, Astore->rowptr, (m_loc+1) * sizeof(int_t));
 }
+#endif	  
 
 /*! \brief Print a summary of the testing results. */
 void
@@ -318,6 +320,7 @@ int main(int argc, char *argv[])
 			options.Fact = fact;
 			if ( fact == SamePattern ) {
 			    // {L,U} not re-used in subsequent call to PDGSSVX.
+				dDestroy_Tree(n, &grid, &LUstruct);  				
 			    Destroy_LU(n, &grid, &LUstruct);
 			}
 
@@ -392,7 +395,8 @@ int main(int argc, char *argv[])
 			ScalePermstruct.DiagScale = NOEQUIL; /* Avoid free R/C again. */
 		    }
 		    ScalePermstructFree(&ScalePermstruct);
-		    Destroy_LU(n, &grid, &LUstruct);
+		    dDestroy_Tree(n, &grid, &LUstruct);  
+			Destroy_LU(n, &grid, &LUstruct);
 		    LUstructFree(&LUstruct);
 		    if ( options.SolveInitialized ) {
 			dSolveFinalize(&options, &SOLVEstruct);
