@@ -54,6 +54,7 @@ pzcompute_resid(int m, int n, int nrhs, SuperMatrix *A,
 		doublecomplex *x, int ldx, doublecomplex *b, int ldb,
 		gridinfo_t *grid, SOLVEstruct_t *SOLVEstruct, double *resid);
 
+#if 0	 
 /*! \brief Copy matrix A into matrix B, in distributed compressed row format. */
 void
 zCopy_CompRowLoc_Matrix_dist(SuperMatrix *A, SuperMatrix *B)
@@ -78,6 +79,7 @@ zCopy_CompRowLoc_Matrix_dist(SuperMatrix *A, SuperMatrix *B)
     memcpy(Bstore->colind, Astore->colind, nnz_loc * sizeof(int_t));
     memcpy(Bstore->rowptr, Astore->rowptr, (m_loc+1) * sizeof(int_t));
 }
+#endif	  
 
 /*! \brief Print a summary of the testing results. */
 void
@@ -317,7 +319,7 @@ int main(int argc, char *argv[])
 			options.Fact = fact;
 			if ( fact == SamePattern ) {
 			    // {L,U} not re-used in subsequent call to PDGSSVX.
-			    Destroy_LU(n, &grid, &LUstruct);
+				zDestroy_Tree(n, &grid, &LUstruct);  					    Destroy_LU(n, &grid, &LUstruct);
 			}
 
 		    } /* end if .. first time factor */
@@ -391,6 +393,7 @@ int main(int argc, char *argv[])
 			ScalePermstruct.DiagScale = NOEQUIL; /* Avoid free R/C again. */
 		    }
 		    ScalePermstructFree(&ScalePermstruct);
+		    zDestroy_Tree(n, &grid, &LUstruct);  
 		    Destroy_LU(n, &grid, &LUstruct);
 		    LUstructFree(&LUstruct);
 		    if ( options.SolveInitialized ) {
